from scan.database.rabbitmq import RabbitMQ
from scan.database.redis import Redis
from scan.database.mongodb import MongoDB
from scan.database.oss import OSS
from scan.database.pg import Postgresql
from scan.database.kafka import Kafka
from scan.database.r2 import CFR2
from scan.database.aior2 import AioCFR2
